package com.sd.business.controller.purchase;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.sd.business.entity.purchase.dto.PurchaseBomImportDataDto;
import com.sd.business.entity.purchase.dto.PurchaseDto;
import com.sd.business.entity.purchase.dto.PurchaseSelectDto;
import com.sd.business.entity.purchase.vo.PurchaseVo;
import com.sd.business.service.purchase.PurchaseService;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;


/**
 * <p>
 * 采购合同 前端控制器
 * </p>
 *
 * @author
 * @since 2023-12-12
 */
@RestController
@RequestMapping("/purchase")
public class PurchaseController {

    @Autowired
    private PurchaseService purchaseService;

    /**
     * 采购合同分页
     */
    @PostMapping("/page")
    public Page<PurchaseVo> page(@RequestBody PurchaseSelectDto dto) {
        return purchaseService.getPage(dto);
    }

    /**
     * 采购合同明细
     */
    @PostMapping("/detail")
    public PurchaseVo detail(@RequestBody BaseSelectDto dto) {
        return purchaseService.detail(dto.getId());
    }

    /**
     * 采购合同新增
     */
    @PostMapping("/add")
    public void add(@RequestBody PurchaseDto dto) {
        dto.setFlowStatus(FlowStatusEnum.READY_START.getKey());
        purchaseService.add(dto);
    }

    /**
     * 采购合同编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody PurchaseDto dto) {
        purchaseService.edit(dto);
    }

    /**
     * 采购合同删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        purchaseService.delete(dto.getId());
    }

    /**
     * 采购终止
     */
    @PostMapping("/purchaseTermination")
    public void purchaseTermination(@RequestBody BaseSelectDto dto) {
        purchaseService.purchaseTermination(dto.getId());
    }

    /**
     * 导入E10采购合同
     */
    @PostMapping("/purchaseImport")
    public void purchaseImport(MultipartFile file) {
        List<PurchaseBomImportDataDto> list = ExcelUtil.read(file, PurchaseBomImportDataDto.class);
        purchaseService.purchaseImport(list);
    }
}
