package com.sd.business.entity.order.enums;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.exception.ServiceException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Getter
@AllArgsConstructor
public enum OrderExceptionTypeEnum {

    NORMAL(0, "正常订单"),
    ORDER_CLOSURE(1, "万里牛状态异常：订单关闭"),
    ORDER_ABORT(2, "万里牛状态异常：订单异常结束"),
    ORDER_EXCEPTION_HANDLING(3, "万里牛状态异常：订单异常处理"),
    UNKNOWN_WAREHOUSE(4, "无法通过仓库编码找到事业部"),
    UNKNOWN_SKU_SPEC(5, "订单存在未知sku规格"),
    UNKNOWN_BOM_SPEC(6, "订单sku规格未绑定主材"),
    UNKNOWN_PRINTING_PAPER_BOM_SPEC(7, "订单sku规格未绑定打印纸"),
    SKU_UPDATE(8, "订单商品被更改"),
    NO_DESIGN_DOCUMENT(9, "未绑定图稿"),

    ;

    private static final Map<Integer, OrderExceptionTypeEnum> map = new HashMap<>();

    static {
        for (OrderExceptionTypeEnum orderExceptionTypeEnum : values()) {
            map.put(orderExceptionTypeEnum.getKey(), orderExceptionTypeEnum);
        }
    }

    private final Integer key;
    private final String value;

    /**
     * 通过key获取名称
     */
    public static OrderExceptionTypeEnum getEnum(Integer key) {
        OrderExceptionTypeEnum orderExceptionTypeEnum = map.get(key);
        if (orderExceptionTypeEnum == null) {
            throw new ServiceException("未知订单异常类型：" + key);
        }
        return orderExceptionTypeEnum;
    }

    public static List<Map<String, Object>> getMapList() {
        return Arrays.stream(values())
                .filter(item -> ObjectUtil.notEqual(NORMAL, item))
                .map(item -> {
                    Map<String, Object> tempMap = new HashMap<>();
                    tempMap.put("key", item.getKey());
                    tempMap.put("value", item.getValue());
                    return tempMap;
                })
                .collect(Collectors.toList());
    }

    /**
     * 通过key获取名称
     */
    public static String getEnumValue(Integer key) {
        return getEnum(key).getValue();
    }

}
