package com.sd.business.entity.work.enums;

import com.ruoyi.common.exception.ServiceException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.math.BigDecimal;

@Getter
@AllArgsConstructor
public enum WorkOrderFixationSpecEnum {

    SPEC_30_40(new BigDecimal("30"), new BigDecimal("40"), 430),
    ;

    /**
     * 长
     */
    private final BigDecimal length;

    /**
     * 宽
     */
    private final BigDecimal width;

    /**
     * 最大数量
     */
    private final int maxNum;

    /**
     * 是否是固定排版
     */
    public static boolean isFixation(BigDecimal length, BigDecimal width) {
        if (length == null) {
            throw new ServiceException("未知长度");
        }
        if (width == null) {
            throw new ServiceException("未知宽度");
        }

        for (WorkOrderFixationSpecEnum value : values()) {
            if (value.length.compareTo(length) == 0 && value.width.compareTo(width) == 0) {
                return true;
            }
        }
        return false;
    }

    public static WorkOrderFixationSpecEnum getEnum(BigDecimal length, BigDecimal width) {
        if (length == null) {
            throw new ServiceException("未知长度");
        }
        if (width == null) {
            throw new ServiceException("未知宽度");
        }

        for (WorkOrderFixationSpecEnum value : values()) {
            if (value.length.compareTo(length) == 0 && value.width.compareTo(width) == 0) {
                return value;
            }
        }
        throw new ServiceException("未知固定排版规格");
    }

}
