package com.sd.business.service.apply.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.sd.business.entity.apply.po.ApplyBuy;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.entity.apply.vo.ApplyBuyBomVo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.mapper.apply.ApplyBuyMapper;
import com.sd.business.service.apply.ApplyBuyBomService;
import com.sd.business.service.apply.ApplyBuyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sd.business.entity.apply.vo.ApplyBuyVo;
import com.sd.business.entity.apply.dto.ApplyBuySelectDto;
import com.sd.business.entity.apply.dto.ApplyBuyDto;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;


/**
 * <p>
 * 申购单 服务实现类
 * </p>
 *
 * @author 
 * @since 2024-01-03
 */
@Service
public class ApplyBuyServiceImpl extends ServiceImpl<ApplyBuyMapper, ApplyBuy> implements ApplyBuyService {

    @Autowired
    private ApplyBuyBomService applyBuyBomService;

    @Override
    public Page<ApplyBuyVo> getPage(ApplyBuySelectDto dto) {

        Page<ApplyBuyVo> page = Sql.create(ApplyBuyVo.class)
                .selectAll(ApplyBuy.class)
                .from(ApplyBuy.class)
                .like(ApplyBuy::getCode, dto.getCode())
                .eq(ApplyBuy::getFlowStatus, dto.getFlowStatus())
                .ge(ApplyBuy::getApplyTime, dto.getBeginTime())
                .le(ApplyBuy::getApplyTime, dto.getEndTime())
                .orderByDesc(ApplyBuy::getId)
                .page(dto);

        return page;
    }

    @Override
    public ApplyBuyVo detail(Long id) {

        ApplyBuyVo vo = Sql.create(ApplyBuyVo.class)
                .selectAll(ApplyBuy.class)
                .from(ApplyBuy.class)
                .eq(ApplyBuy::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        List<ApplyBuyBomVo> bomVoList = Sql.create(ApplyBuyBomVo.class)
                .selectAll(ApplyBuyBom.class)
                .selectAs(BomSpec::getCode, ApplyBuyBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, ApplyBuyBomVo::getBomSpecName)
                .selectAs(BomSpec::getColour, ApplyBuyBomVo::getBomSpecColour)
                .selectAs(BomSpec::getLength, ApplyBuyBomVo::getBomSpecLength)
                .selectAs(BomSpec::getWidth, ApplyBuyBomVo::getBomSpecWidth)
                .selectAs(BomSpec::getHeight, ApplyBuyBomVo::getBomSpecHeight)
                .from(ApplyBuyBom.class)
                .innerJoin(BomSpec.class, BomSpec::getId, ApplyBuyBom::getBomSpecId)
                .eq(ApplyBuyBom::getId, id)
                .list();

        vo.setApplyBuyBomList(bomVoList);

        return vo;
    }

    @Override
    public void add(ApplyBuyDto dto) {
        Long applyBuyId = IdWorker.getId();

        dto.setId(applyBuyId);
        dto.setCode(CodeEnum.APPLY_BUY_CODE.getCode());

        List<ApplyBuyBom> applyBuyBomList = dto.getApplyBuyBomList();
        applyBuyBomList.forEach(item -> {
            item.setApplyBuyId(applyBuyId);
            item.setPurchaseQuantity(BigDecimal.ZERO);
            item.setFrozenQuantity(BigDecimal.ZERO);
        });

        save(dto);
        applyBuyBomService.saveBatch(applyBuyBomList);
    }

    @Override
    public void edit(ApplyBuyDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

}
