package com.sd.business.service.in.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageBomSelectDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.in.vo.InOutStorageBomErpExportVo;
import com.sd.business.entity.in.vo.InOutStorageBomVo;
import com.sd.business.entity.in.vo.InOutStorageErpExportVo;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.mapper.in.InOutStorageBomMapper;
import com.sd.business.service.in.InOutStorageBomService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.framework.util.TemplateExcelUtil;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import static java.util.Comparator.comparing;


/**
 * <p>
 * 出入库bom 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class InOutStorageBomServiceImpl extends ServiceImpl<InOutStorageBomMapper, InOutStorageBom> implements InOutStorageBomService {

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private HttpServletResponse response;

    @Override
    public Page<InOutStorageBomVo> getPage(InOutStorageBomSelectDto dto) {

        Page<InOutStorageBomVo> page = Sql.create(InOutStorageBomVo.class)
                .selectAll(InOutStorageBom.class)
                .select(InOutStorage::getInOutStorageTime,
                        InOutStorage::getDepartmentId,
                        InOutStorage::getType,
                        InOutStorage::getDetailType,
                        InOutStorage::getCode,
                        InOutStorage::getApplicant,
                        InOutStorage::getRemark)
                .selectAs(BomSpec::getCode, InOutStorageBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, InOutStorageBomVo::getBomSpecName)
                .selectAs(Warehouse::getType, InOutStorageBomVo::getWarehouseType)
                .selectAs(Warehouse::getName, InOutStorageBomVo::getWarehouseName)
                .selectAs(Warehouse::getCode, InOutStorageBomVo::getWarehouseCode)
                .selectAs(Department::getName, InOutStorageBomVo::getDepartmentName)
                .selectAs(Purchase::getCode, InOutStorageBomVo::getPurchaseCode)

                .from(InOutStorageBom.class)
                .leftJoin(InOutStorage.class, InOutStorageBom::getInOutStorageId)
                .leftJoin(BomSpec.class, InOutStorageBom::getBomSpecId)
                .leftJoin(Warehouse.class, InOutStorage::getWarehouseId)
                .leftJoin(Department.class, InOutStorage::getDepartmentId)
                .leftJoin(Purchase.class, InOutStorage::getPurchaseId)

                .orderByDesc(InOutStorageBom::getId)
                .like(InOutStorage::getCode, dto.getCode())
                .eq(InOutStorage::getDepartmentId, dto.getDepartmentId())
                .eq(InOutStorage::getType, dto.getType())
                .eq(InOutStorage::getDetailType, dto.getDetailType())
                .ge(InOutStorage::getInOutStorageTime, dto.getBeginTime())
                .le(InOutStorage::getInOutStorageTime, dto.getEndTime())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getName, dto.getBomSpecName())
                .notIn(BomSpec::getCode, "40904003", "409001", "40101010") // 筛选吊牌、不干胶、彩纸出入库记录
                .like(Purchase::getCode, dto.getPurchaseCode())
                .eq(Warehouse::getId, dto.getWarehouseId())

                .page(dto);

        List<InOutStorageBomVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        records.forEach(item -> {
            if (item.getDepartmentId() == 0L) {
                item.setDepartmentName("胜德体育");
            }
        });

        return page;
    }

    @Override
    public void exportErpExcel(InOutStorageBomSelectDto dto) {
        dto.setSearchAll(true);
        Page<InOutStorageBomVo> page = this.getPage(dto);
        List<InOutStorageBomVo> list = page.getRecords();
        List<InOutStorageBomVo> inOutStorageBomList = list.stream()
                .filter(item -> {
                    if (!(ObjectUtil.equals(item.getWarehouseCode(), "509")
                            || ObjectUtil.equals(item.getWarehouseCode(), "503"))) {
                        return false;
                    }
                    if (dto.getDetailType() != null) {
                        return true;
                    }
                    // 数据筛选，出库导出生产出库，入库导出退货和归还
                    if (ObjectUtil.equals(dto.getType(), InOutTypeEnum.IN.getKey())
                            && (ObjectUtil.equals(item.getDetailType(), InDetailTypeEnum.RETURN_GOODS.getKey())
                            || ObjectUtil.equals(item.getDetailType(), InDetailTypeEnum.GIVE_BACK.getKey()))) {
                        return true;
                    } else return ObjectUtil.equals(dto.getType(), InOutTypeEnum.OUT.getKey())
                            && ObjectUtil.equals(item.getDetailType(), OutDetailTypeEnum.PRODUCTION.getKey());
                })
                .collect(Collectors.toList());
        // 查询详情信息
        Set<Long> inOutStorageIds = inOutStorageBomList.stream().map(InOutStorageBom::getInOutStorageId).collect(Collectors.toSet());
        if (ObjectUtil.isEmpty(inOutStorageIds)) {
            inOutStorageIds.add(-1L);
        }
        List<InOutStorage> inOutStorageList = inOutStorageService.list(q -> q
                .in(BaseIdPo::getId, inOutStorageIds)
                .orderByAsc(InOutStorage::getInOutStorageTime));

        List<InOutStorageErpExportVo> inOutStorageErpExportVoList = new ArrayList<>();
        String dateFormat = "";
        for (InOutStorage inOutStorage : inOutStorageList) {
            InOutStorageErpExportVo inOutStorageErpExportVo = new InOutStorageErpExportVo();
            inOutStorageErpExportVo.setId(inOutStorage.getId());
            inOutStorageErpExportVo.setPlantCode("SDTY");
            if (Objects.equals(inOutStorage.getType(), InOutTypeEnum.OUT.getKey())) {
                inOutStorageErpExportVo.setDocCode("T118");
            } else {
                inOutStorageErpExportVo.setDocCode("T119");
            }
            // 获取单号 系统单号 XX-yyMMdd-000001
            String[] codes = inOutStorage.getCode().split("-");
            String substring = codes[1].substring(0, codes[1].length() - 2);
            String format = "-20" + substring;
            if (!Objects.equals(dateFormat, format)) {
                dateFormat = format;
            }
            // 后四位生成规则，取系统单号 dd01
            int codeNum = Integer.parseInt(codes[1].substring(codes[1].length() - 2) + codes[2].substring(codes[2].length() - 2));
            String code = inOutStorageErpExportVo.getDocCode() + format + String.format("%04d", codeNum);
            inOutStorageErpExportVo.setCode(code);
            inOutStorageErpExportVo.setCreateTime(DateUtil.format(inOutStorage.getInOutStorageTime(), "yyyy/MM/dd HH:mm:ss"));
            inOutStorageErpExportVo.setDeptCode("STWH");
            inOutStorageErpExportVo.setEmployeeCode("SKWH002");

            inOutStorageErpExportVoList.add(inOutStorageErpExportVo);
        }

        Map<Long, String> map = inOutStorageErpExportVoList.stream()
                .collect(Collectors.toMap(InOutStorageErpExportVo::getId, InOutStorageErpExportVo::getCode));
        Map<String, Integer> serialNumberMap = new HashMap<>();
        List<InOutStorageBomErpExportVo> inOutStorageBomErpExportVoList = inOutStorageBomList.stream()
                .map(item -> {
                    InOutStorageBomErpExportVo inOutStorageBomErpExportVo = new InOutStorageBomErpExportVo();
                    inOutStorageBomErpExportVo.setCode(map.get(item.getInOutStorageId()));
                    inOutStorageBomErpExportVo.setBomSpecCode(item.getBomSpecCode());
                    inOutStorageBomErpExportVo.setQuantity(item.getQuantity().toPlainString());
                    inOutStorageBomErpExportVo.setWarehouseCode(item.getWarehouseCode());
                    inOutStorageBomErpExportVo.setUnit("PCS");
                    inOutStorageBomErpExportVo.setPieces("0");
                    inOutStorageBomErpExportVo.setSourceRtk("OTHER");
                    //if (item.getUnitPrice() == null) {
                    //    item.setUnitPrice(BigDecimal.ZERO);
                    //}
                    //inOutStorageBomErpExportVo.setUnitCost(item.getUnitPrice().toPlainString());
                    //inOutStorageBomErpExportVo.setCostAmt(item.getUnitPrice().multiply(item.getQuantity()).toPlainString());
                    return inOutStorageBomErpExportVo;
                })
                .sorted(comparing(InOutStorageBomErpExportVo::getCode))
                .peek(item -> {
                    Integer serialNumber = serialNumberMap.get(item.getCode());
                    if (serialNumber == null) {
                        serialNumber = 1;
                        serialNumberMap.put(item.getCode(), serialNumber);
                    } else {
                        serialNumberMap.put(item.getCode(), serialNumber += 1);
                    }
                    item.setSerialNumber(serialNumber.toString());
                }).collect(Collectors.toList());
        String fileName;
        if (Objects.equals(dto.getType(), 1)) {
            fileName = "入库单";
        } else {
            fileName = "出库单";
        }
        TemplateExcelUtil.writeBrowserMultiSheet("inOutStorageDetails.xlsx", 2, fileName, response, inOutStorageErpExportVoList, inOutStorageBomErpExportVoList);
    }

}
