package com.sd.business.service.in.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.dto.InOutStorageSelectDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.in.vo.InOutStorageVo;
import com.sd.business.mapper.in.InOutStorageMapper;
import com.sd.business.service.in.InOutStorageBomService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.purchase.PurchaseService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;


/**
 * <p>
 * 出入库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class InOutStorageServiceImpl extends ServiceImpl<InOutStorageMapper, InOutStorage> implements InOutStorageService {

    @Autowired
    private InOutStorageBomService inOutStorageBomService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private PurchaseService purchaseService;

    @Override
    public Page<InOutStorageVo> getPage(InOutStorageSelectDto dto) {

        Page<InOutStorageVo> page = Sql.create(InOutStorageVo.class)
                .selectAll(InOutStorage.class)
                .from(InOutStorage.class)
                .orderByDesc(InOutStorage::getId)
                .page(dto);

        return page;
    }

    @Override
    public InOutStorageVo detail(Long id) {

        InOutStorageVo vo = Sql.create(InOutStorageVo.class)
                .selectAll(InOutStorage.class)
                .from(InOutStorage.class)
                .eq(InOutStorage::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(InOutStorageDto dto) {
        dto.setId(IdWorker.getId());
        dto.setInOutStorageTime(ObjectUtil.defaultIfNull(dto.getInOutStorageTime(), new Date()));

        List<InOutStorageBom> inOutStorageBomList = dto.getInOutStorageBomList();
        inOutStorageBomList.forEach(item -> item.setInOutStorageId(dto.getId()));

        if (InOutTypeEnum.getInOutType(dto.getType()).equals(InOutTypeEnum.IN)) {
            if (InDetailTypeEnum.PURCHASE.getKey().equals(dto.getDetailType())) {
                // 更新采购到货数量
                purchaseService.updatePurchaseBomArrivalQuantity(dto.getPurchaseId(), inOutStorageBomList);
            }
            // 赋值入库单号
            dto.setCode(CodeEnum.IN_CODE.getCode());
            // 更新库存
            inventoryService.in(dto);
        } else {
            // 赋值出库单号
            dto.setCode(CodeEnum.OUT_CODE.getCode());
            // 更新库存
            inventoryService.out(dto);
        }

        // 保存出入库信息
        this.save(dto);
        inOutStorageBomService.saveBatch(inOutStorageBomList);
    }

    @Override
    public void edit(InOutStorageDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

}
