package com.sd.business.service.order.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.annotation.LogicIgnore;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.dto.OrderPackageBomDto;
import com.sd.business.entity.order.dto.OrderSkuDto;
import com.sd.business.entity.order.dto.SkuSpecPackageBomDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.order.po.OrderSkuProductionCost;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderPackageBomVo;
import com.sd.business.entity.order.vo.OrderSkuBomVo;
import com.sd.business.entity.order.vo.OrderSkuExportVo;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.order.vo.OutBomVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderInfoMapper;
import com.sd.business.service.bom.BomService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.order.OrderInfoService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuProductionCostService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.excel.util.ExcelUtil;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.sql.Where;
import com.sd.mq.config.ArtworkConfig;
import com.sd.mq.entity.TempArtworkMessage;
import com.sd.mq.util.RabbitMqUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class OrderInfoServiceImpl extends ServiceImpl<OrderInfoMapper, OrderInfo> implements OrderInfoService {

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderSkuProductionCostService orderSkuProductionCostService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomService bomService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private PriceBillingStandardService priceBillingStandardService;

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Autowired
    private SkuSpecLinkService skuSpecLinkService;

    @Autowired
    private HttpServletResponse response;

    @Override
    public Page<OrderInfoVo> getPage(OrderInfoSelectDto dto) {

        Where<OrderInfoVo> sql = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)

                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)

                .orderByDesc("if(t1.status = " + OrderStatusEnum.SUSPEND.getKey() + ", 1, 0)")
                .orderByDesc(OrderInfo::getId)
                .eq(OrderInfo::getStatus, dto.getStatus())
                .eq(OrderInfo::getSettlementStatus, dto.getSettlementStatus())
                .eq(OrderInfo::getFlowStatus, dto.getFlowStatus())
                .like(OrderInfo::getCode, dto.getCode())
                .like(OrderInfo::getWlnCode, dto.getWlnCode())
                .ge(OrderInfo::getWlnCreateTime, dto.getBeginTime())
                .le(OrderInfo::getWlnCreateTime, dto.getEndTime())
                .findInSet(OrderInfo::getExceptionType, dto.getExceptionType())
                .eq(Department::getId, dto.getDepartmentId())
                .like(Department::getName, dto.getDepartmentName());

        List<Long> detpIdList = sysUserService.getDetpIdListByUserId(SecurityUtils.getUserId());
        if (!detpIdList.isEmpty()) {
            sql.in(OrderInfo::getDepartmentId, detpIdList);
            sql.eq(OrderInfo::getCreateUser, SecurityUtils.getUserId());
        }

        if (StatusConstant.YES.equals(dto.getLinkedStatementOfAccount())) {
            sql.isNotNull(OrderInfo::getStatementOfAccountId);
            sql.in(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        } else if (StatusConstant.NO.equals(dto.getLinkedStatementOfAccount())) {
            sql.isNull(OrderInfo::getStatementOfAccountId);
            sql.in(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        }

        if (StatusConstant.YES.equals(dto.getException())) {
            sql.ne(OrderInfo::getExceptionType, StatusConstant.NO);
        } else if (StatusConstant.NO.equals(dto.getException())) {
            sql.eq(OrderInfo::getExceptionType, StatusConstant.NO);
        }

        Page<OrderInfoVo> page = sql.page(dto);
        List<OrderInfoVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        for (OrderInfoVo record : records) {
            // 设置异常提示
            String exceptionType = record.getExceptionType();
            if (exceptionType.equals("0")) {
                continue;
            }
            String exceptionTypeDetail = Arrays.stream(exceptionType.split(","))
                    .map(Convert::toInt)
                    .map(OrderExceptionTypeEnum::getEnumValue)
                    .collect(Collectors.joining(","));
            record.setExceptionTypeDetail(exceptionTypeDetail);
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {

        OrderInfoVo vo = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)
                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)
                .eq(OrderInfo::getId, id)
                .one();

        Assert.notNull(vo, "未知订单");

        // 订单sku列表
        List<OrderSkuVo> orderSkuList = Sql.create(OrderSkuVo.class)
                .selectAll(OrderSku.class)
                .selectAs(SkuSpec::getCode, OrderSkuVo::getSkuSpecCode)
                .selectAs(SkuSpec::getName, OrderSkuVo::getSkuSpecName)
                .selectAs(BomSpec::getCode, OrderSkuVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuVo::getBomSpecName)
                .from(OrderSku.class)
                .leftJoin(SkuSpec.class, OrderSku::getSkuSpecId)
                .leftJoin(BomSpec.class, OrderSku::getBomSpecId)
                .eq(OrderSku::getOrderId, id)
                .list();

        // 包材配件/单品
        List<OrderSkuBomVo> orderSkuBomList = Sql.create(OrderSkuBomVo.class)
                .selectAll(OrderSkuBom.class)
                .selectAs(BomSpec::getCode, OrderSkuBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuBomVo::getBomSpecName)
                .selectAs(Bom::getBomClassifyId, OrderSkuBomVo::getBomClassifyId)
                .from(OrderSkuBom.class)
                .leftJoin(BomSpec.class, OrderSkuBom::getBomSpecId)
                .leftJoin(Bom.class, BomSpec::getBomId)
                .eq(OrderSkuBom::getOrderId, id)
                .list();

        // 包装明细
        List<OrderPackageBomVo> orderPackageBomList = Sql.create(OrderPackageBomVo.class)
                .selectAll(OrderPackageBom.class)
                .selectAs(BomSpec::getCode, OrderPackageBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderPackageBomVo::getBomSpecName)
                .from(OrderPackageBom.class)
                .leftJoin(BomSpec.class, OrderPackageBom::getBomSpecId)
                .eq(OrderPackageBom::getOrderId, id)
                .list();

        // 赋值
        Map<Long, List<OrderSkuBomVo>> map = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBomVo::getOrderSkuId));
        orderSkuList.forEach(item -> item.setOrderSkuBomList(map.getOrDefault(item.getId(), Collections.emptyList())));
        vo.setOrderSkuList(orderSkuList);
        vo.setOrderPackageBomList(orderPackageBomList);

        return vo;
    }

    @DSTransactional
    @Override
    public void add(OrderInfoDto dto) {
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();

        // 新增订单
        dto.setSource(1);
        dto.setCode("PI" + new Date().getTime());
        dto.setSettlementStatus(1);
        dto.setTaxRate(BigDecimal.TEN);
        dto.setTag("0");
        dto.setExceptionType(OrderExceptionTypeEnum.NORMAL.getKey().toString());

        // 拥有采购角色的账号创建的自主订单为采购订单
        if (Objects.equals(dto.getType(), 1)) {
            Long userId = SecurityUtils.getUserId();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        // 订单为委外订单时，订单分类为委外订单
        else if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());

            // 委外订单修改金额
            dto.setProductTotalAmount(BigDecimal.ZERO);
            dto.setDeliveryMaterialsFee(BigDecimal.ZERO);
            dto.setManagementFee(BigDecimal.ZERO);
            dto.setPackingLabor(dto.getPackingLabor().multiply(new BigDecimal(2)));
            dto.setTotalAmount(dto.getProductTotalAmount()
                    .add(dto.getCustomProcessingFee())
                    .add(dto.getLssueFee())
                    .add(dto.getDeliveryMaterialsFee())
                    .add(dto.getPackingLabor())
                    .add(dto.getPackagingMaterialCost())
                    .add(dto.getManagementFee()));
        }

        save(dto);

        // 新增订单产品包材配件
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> {
                    // 委外订单修改金额
                    if (ObjectUtil.equals(dto.getType(), 2)) {
                        item.setUnitPrice(BigDecimal.ZERO);
                        item.setLssueFee(BigDecimal.ZERO);
                        item.setDeliveryMaterialsFee(BigDecimal.ZERO);
                        item.setManagementFee(BigDecimal.ZERO);
                        item.setPackingLabor(item.getPackingLabor().multiply(new BigDecimal(2)));
                    }
                })
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.saveBatch(tempOrderSkuList);

        // 新增订单产品包装信息
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.saveBatch(orderSkuBomList);

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), new ArrayList<>());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee().multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(orderSkuProductionCost.getMaterialCost()
                    .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                    .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                    .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.saveBatch(orderSkuProductionCostList);

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.notEqual(dto.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.equal(dto.getFlowStatus(), FlowStatusEnum.PASS.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.addByOrderId(dto.getId());
            update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 保存订单包装信息
        if (ObjectUtil.isNotEmpty(dto.getOrderPackageBomList())) {
            List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(dto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.saveBatch(orderPackageBomList);
        }

        // 保存产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.saveFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 保存附件
        ObsFileUtil.saveFile(dto.getFileList(), dto.getId());
    }

    @DSTransactional
    @Override
    public void edit(OrderInfoDto dto) {
        // 拥有采购角色的账号创建的自主订单为采购订单
        if (Objects.equals(dto.getType(), 1)) {
            Long userId = dto.getCreateUser();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        // 订单为委外订单时，订单分类为委外订单
        else if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
        }

        // 更新订单
        updateById(dto);

        // 更新订单产品
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新订单产品包材配件
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), new ArrayList<>());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.editLinked(orderSkuProductionCostList, OrderSkuProductionCost::getOrderId, dto.getId());

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.equals(dto.getStatus(), OrderStatusEnum.SCHEDULING.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.addByOrderId(dto.getId());
            update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 更新订单产品包装信息
        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        if (tempOrderPackageBomList == null) {
            tempOrderPackageBomList = new ArrayList<>();
        }
        List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 更新订单产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 更新订单附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());
    }

    @DSTransactional
    @Override
    public void delete(Long id) {
        removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
        orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));
        orderSkuProductionCostService.remove(q -> q.eq(OrderSkuProductionCost::getOrderId, id));
        productionWorkOrderService.remove(q -> q.eq(ProductionWorkOrder::getOrderId, id));

        ObsFileUtil.removeFile(id);
    }

    @DSTransactional
    @Override
    public synchronized void deleteAndStore(Long id) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        if (orderInfo.getStatus() <= OrderStatusEnum.SCHEDULING.getKey()) {
            delete(id);
            return;
        }

        // 获取订单商品主材和包材
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));

        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        // 合并主材和包材
        List<InOutStorageBom> inOutStorageBomList = orderSkuList.stream().map(item -> {
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(item.getBomSpecId());
            inOutStorageBom.setQuantity(item.getQuantity());
            return inOutStorageBom;
        }).collect(Collectors.toList());

        for (OrderSkuBom orderSkuBom : orderSkuBomList) {
            OrderSku orderSku = orderSkuMap.get(orderSkuBom.getOrderSkuId());
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(orderSkuBom.getBomSpecId());
            inOutStorageBom.setQuantity(orderSkuBom.getQuantity().multiply(orderSku.getQuantity()));
            inOutStorageBomList.add(inOutStorageBom);
        }

        // 合并数量
        Map<Long, InOutStorageBom> map = inOutStorageBomList.stream().collect(Collectors.toMap(
                InOutStorageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }
        ));
        inOutStorageBomList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(map.keySet());

        // 主材从半成品仓入库
        semiFinishedProductOutStorage(inOutStorageBomList, bomSpecBoMap, orderInfo);

        // 包材从包材仓出库
        packagingMaterialOutStorage(inOutStorageBomList, bomSpecBoMap, orderInfo);

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        delete(id);
    }

    @DSTransactional
    @Override
    public void deleteValidated(Long id) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)
                && orderInfo.getStatus() > OrderStatusEnum.SCHEDULING.getKey()) {
            throw new ServiceException("订单正在生产中，无法直接删除，需要走审批流程");
        }

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        // 删除订单以及订单关联数据
        delete(id);
    }

    @DSTransactional
    @Override
    public void confirmation(OrderInfoDto dto) {
        OrderInfo order = getById(dto.getId());

        if (ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.UNDER_REVIEW.getKey())) {
            throw new ServiceException("订单不在草稿或待确认状态");
        }

        // 自主订单需要锁定库存
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();

        // 更新订单
        dto.setStatus(OrderStatusEnum.SCHEDULING.getKey());
        // 订单为委外订单时，订单分类为委外订单，订单费用修改
        if (Objects.equals(dto.getType(), 2)) {
            dto.setClassify(OrderClassifyEnum.OUTSOURCE_ORDER.getKey());
            if (!Objects.equals(dto.getSource(), 1)) {
                dto.setLssueFee(BigDecimal.ZERO);
            }
            dto.setProductTotalAmount(BigDecimal.ZERO);
            dto.setDeliveryMaterialsFee(BigDecimal.ZERO);
            dto.setManagementFee(BigDecimal.ZERO);
            dto.setPackingLabor(dto.getPackingLabor().multiply(new BigDecimal(2)));
            dto.setTotalAmount(dto.getProductTotalAmount()
                    .add(dto.getCustomProcessingFee())
                    .add(dto.getLssueFee())
                    .add(dto.getDeliveryMaterialsFee())
                    .add(dto.getPackingLabor())
                    .add(dto.getPackagingMaterialCost())
                    .add(dto.getManagementFee()));
            // 清空订单快递包材
            dto.setOrderPackageBomList(null);
        } else if (Objects.equals(dto.getType(), 1)) {
            // 拥有采购角色的账号创建的自主订单为采购订单
            Long userId = dto.getCreateUser();
            Set<String> roleKeys = roleService.selectRolePermissionByUserId(userId);
            if (!Collections.disjoint(roleKeys, Arrays.asList("sypurchasing", "purchasingOfficer", "bzpurchasing"))) {
                dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());
            }
        }
        updateById(dto);

        // 更新订单产品
        List<OrderSku> tempOrderSkuList = orderSkuList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .peek(item -> {
                    // 委外订单修改金额
                    if (ObjectUtil.equals(dto.getType(), 2)) {
                        item.setUnitPrice(BigDecimal.ZERO);
                        item.setLssueFee(BigDecimal.ZERO);
                        item.setDeliveryMaterialsFee(BigDecimal.ZERO);
                        item.setManagementFee(BigDecimal.ZERO);
                        item.setPackingLabor(item.getPackingLabor().multiply(new BigDecimal(2)));
                    }
                })
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
        orderSkuService.editLinked(tempOrderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新包材
        List<OrderSkuBom> orderSkuBomList = orderSkuList.stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 新增订单产品生产成本
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        List<OrderSkuProductionCost> orderSkuProductionCostList = tempOrderSkuList.stream().map(item -> {
            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), new ArrayList<>());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity()
                            .multiply(orderSkuBom.getUnitPrice())
                            .multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee()
                    .multiply(item.getQuantity()));
            orderSkuProductionCost.setTotalAmount(
                    orderSkuProductionCost.getMaterialCost()
                            .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                            .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                            .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
        orderSkuProductionCostService.editLinked(orderSkuProductionCostList, OrderSkuProductionCost::getOrderId, dto.getId());

        // 是委外订单并且没有包材时，修改订单状态为生产中，并生成生产任务和工单
        if (ObjectUtil.equals(dto.getStatus(), OrderStatusEnum.SCHEDULING.getKey())
                && ObjectUtil.equals(dto.getType(), 2)
                && ObjectUtil.isEmpty(orderSkuBomList)) {
            dto.setStatus(OrderStatusEnum.IN_PRODUCTION.getKey());
            // 生成工单
            productionWorkOrderService.addByOrderId(dto.getId());
            update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey()).eq(BaseIdPo::getId, dto.getId()));
        }

        // 更新订单产品包装信息
        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        if (tempOrderPackageBomList == null) {
            tempOrderPackageBomList = new ArrayList<>();
        }
        List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 更新产品不干胶图稿
        for (OrderSkuDto item : orderSkuList) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

        // 更新附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());

        // 发送mq下载临时图稿
        sendMq(dto);
    }

    @Override
    public SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto) {
        // 查询单价
        Long skuSpecId = dto.getSkuSpecId();
        SkuSpec skuSpec = skuSpecService.getById(skuSpecId);
        Assert.notNull(skuSpec, "未知sku规格");
        Long bomSpecId = skuSpec.getBomSpecId();
        Assert.notNull(bomSpecId, "sku未绑定bom");
        BomSpec bomSpec = bomSpecService.getById(bomSpecId);
        Assert.notNull(bomSpec, "未知bom规格");
        Bom bom = bomService.getById(bomSpec.getBomId());
        Assert.notNull(bom, "未知bom");

        // 其他价格查询
        Long deptId = SecurityUtils.getDeptId();
        Department department = departmentService.getById(deptId);
        Assert.notNull(department, "未知事业部");
        String priceBillingStandardIdStr = department.getPriceBillingStandardId();
        Assert.notBlank(priceBillingStandardIdStr, "事业部未绑定加工计费标准");

        List<String> priceBillingStandardIdList = Arrays.stream(priceBillingStandardIdStr.split(",")).collect(Collectors.toList());
        List<PriceBillingStandard> priceBillingStandardList = priceBillingStandardService.listByIds(priceBillingStandardIdList);

        List<PriceBillingStandardDetail> priceBillingStandardDetailList = null;

        if (StrUtil.isBlank(bom.getSpecies())) {
            priceBillingStandardDetailList = Collections.emptyList();
        } else {
            for (PriceBillingStandard priceBillingStandard : priceBillingStandardList) {
                if (Objects.equals(bom.getSpecies(), priceBillingStandard.getSpecies())) {
                    priceBillingStandardDetailList = priceBillingStandardDetailService.list(
                            q -> q.eq(PriceBillingStandardDetail::getPriceBillingStandardId, priceBillingStandard.getId()));
                    break;
                }
            }

            if (priceBillingStandardDetailList == null) {
                throw new ServiceException("没有找到加工计费标准");
            }
        }

        // 赋值
        SkuSpecPriceVo skuSpecPriceVo = new SkuSpecPriceVo();
        skuSpecPriceVo.setUnitPrice(bomSpec.getInternalSellingPrice());
        skuSpecPriceVo.setCustomProcessingFee(BigDecimal.ZERO);
        skuSpecPriceVo.setLssueFee(BigDecimal.ZERO);
        skuSpecPriceVo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        skuSpecPriceVo.setPackingLabor(BigDecimal.ZERO);
        skuSpecPriceVo.setManagementFee(BigDecimal.ZERO);

        // TODO 根据收费项目赋值价格
        for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
            String chargeItem = priceBillingStandardDetail.getChargeItem();
            BigDecimal chargePrice = ObjectUtil.defaultIfNull(priceBillingStandardDetail.getChargePrice(), BigDecimal.ZERO);
            switch (chargeItem) {
                case "40":
                    skuSpecPriceVo.setPackingLabor(chargePrice);
                    break;
                case "50":
                    skuSpecPriceVo.setLssueFee(chargePrice);
                    break;
                case "60":
                    skuSpecPriceVo.setDeliveryMaterialsFee(chargePrice);
                    break;
                case "70":
                    skuSpecPriceVo.setManagementFee(chargePrice);
                    break;
                default:
                    //if (Objects.equals(chargeItem, skuSpec.getMachinedPanel())
                    //        && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMin()) >= 0
                    //        && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMax()) < 0) {
                    //    skuSpecPriceVo.setCustomProcessingFee(chargePrice);
                    //    skuSpecPriceVo.setCustomProcessingType(chargeItem);
                    //}
            }
        }

        return skuSpecPriceVo;
    }

    @Override
    public void turnToRegularOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        String exceptionType = orderInfo.getExceptionType();

        if (Objects.equals(exceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())) {
            return;
        }

        for (String s : exceptionType.split(",")) {
            OrderExceptionTypeEnum orderExceptionTypeEnum = OrderExceptionTypeEnum.getEnum(Convert.toInt(s));
            if (ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_CLOSURE)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_ABORT)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_EXCEPTION_HANDLING)) {
                throw new ServiceException(orderExceptionTypeEnum.getValue() + "，无法转为正常订单");
            }
        }

        update(q -> q
                .eq(BaseIdPo::getId, orderInfo.getId())
                .set(OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        // 转trade
        if (Arrays.asList(exceptionType.split(",")).contains(OrderExceptionTypeEnum.ORDER_CLOSURE.getKey().toString())) {
            // 订单分类修改为无理由订单
            update(q -> q
                    .eq(BaseIdPo::getId, orderInfo.getId())
                    .set(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey())
            );
        }
    }

    @Override
    public String getOrderDeliveryCode() {
        return CodeEnum.SH_CODE.getCode();
    }

    @Override
    public void saveOrderDeliveryCode(OrderInfoDto dto) {
        OrderInfo orderInfo = getById(dto.getId());
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        if (StrUtil.isBlank(dto.getDeliveryCode())) {
            throw new ServiceException("送货单号不能为空");
        }
        OrderInfo one = getOne(q -> q.eq(OrderInfo::getDeliveryCode, dto.getDeliveryCode()));
        if (one != null) {
            throw new ServiceException("送货单号重复，请重新生成！");
        }
        update(q -> q.set(OrderInfo::getDeliveryCode, dto.getDeliveryCode()).eq(BaseIdPo::getId, dto.getId()));
    }

    @Override
    public void suspendOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.SUSPEND.getKey()).eq(BaseIdPo::getId, id));
    }

    @Override
    public void cancelSuspendOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.SCHEDULING.getKey()).eq(BaseIdPo::getId, id));
    }

    @Override
    public List<OutBomVo> returnBomList(Long id) {
        OrderInfo orderInfo = getById(id);
        Assert.notNull(orderInfo, "未找到订单");

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));
        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        if (Objects.equals(orderInfo.getClassify(), OrderClassifyEnum.OUTSOURCE_ORDER.getKey())) {
            orderSkuList = Collections.emptyList();
        }

        Map<Long, OutBomVo> map = Stream.concat(
                        // 主材
                        orderSkuList.stream().map(item -> {
                            OutBomVo outBomVo = new OutBomVo();
                            outBomVo.setBomSpecId(item.getBomSpecId());
                            outBomVo.setOutQuantity(item.getQuantity());
                            return outBomVo;
                        }),

                        // 包材
                        orderSkuBomList.stream().map(item -> {
                            OrderSku orderSku = orderSkuMap.get(item.getOrderSkuId());
                            OutBomVo outBomVo = new OutBomVo();
                            outBomVo.setBomSpecId(item.getBomSpecId());
                            outBomVo.setOutQuantity(item.getQuantity().multiply(orderSku.getQuantity()));
                            return outBomVo;
                        })
                )
                .collect(Collectors.toMap(
                        OutBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setOutQuantity(v1.getOutQuantity().add(v2.getOutQuantity()));
                            return v1;
                        })
                );

        // 合并sku主材和包材
        List<OutBomVo> outBomVoList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(map.keySet());

        for (OutBomVo outBomVo : outBomVoList) {
            Long bomSpecId = outBomVo.getBomSpecId();
            BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
            outBomVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
            outBomVo.setBomSpecName(bomSpecBo.getBomSpecName());
            outBomVo.setClassifyId(bomSpecBo.getClassifyId());
            outBomVo.setClassifyName(bomSpecBo.getClassifyName());
            outBomVo.setClassifyParentId(bomSpecBo.getClassifyParentId());

            // 主材
            if (Objects.equals(bomSpecBo.getClassifyParentId(), 1L)) {
                outBomVo.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
                outBomVo.setWarehouseName("半成品仓");
            }
            // 包材
            else {
                outBomVo.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
                outBomVo.setWarehouseName("包材仓");
            }
        }
        return outBomVoList.stream()
                .sorted(Comparator.comparing(OutBomVo::getWarehouseId, Comparator.reverseOrder()).thenComparing(OutBomVo::getBomSpecCode))
                .collect(Collectors.toList());
    }

    @Override
    public void editOrderPackageBom(OrderInfoDto dto) {
        OrderInfo orderInfo = getById(dto.getId());
        Assert.notNull(orderInfo, "未找到订单");

        // 生产中和生产完成的订单快递包材归还入库
        if (ObjectUtil.equals(orderInfo.getStatus(), OrderStatusEnum.IN_PRODUCTION.getKey())
                || ObjectUtil.equals(orderInfo.getStatus(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey())) {
            dto.setCode(orderInfo.getCode());
            dto.setWlnCode(orderInfo.getWlnCode());
            orderPackageBomInOutStorage(dto);
        }

        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();
        BigDecimal totalAmount = orderInfo.getTotalAmount();
        BigDecimal oldDeliveryMaterialsFee = orderInfo.getDeliveryMaterialsFee();
        BigDecimal newDeliveryMaterialsFee = BigDecimal.ZERO;
        if (ObjectUtil.isEmpty(tempOrderPackageBomList)) {
            orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, orderInfo.getId()));
        } else {
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(dto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderInfo.getId());
            newDeliveryMaterialsFee = orderPackageBomList.stream()
                    .map(item -> item.getInternalSellingPrice().multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        orderInfo.setDeliveryMaterialsFee(newDeliveryMaterialsFee);
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));
        BigDecimal quantity = orderSkuList.stream().map(OrderSku::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
        orderSkuList.forEach(item ->
                item.setDeliveryMaterialsFee(orderInfo.getDeliveryMaterialsFee()
                        .divide(quantity, 4, RoundingMode.HALF_UP))
        );
        orderInfo.setTotalAmount(totalAmount.subtract(oldDeliveryMaterialsFee).add(newDeliveryMaterialsFee));
        updateById(orderInfo);
        orderSkuService.updateBatchById(orderSkuList);
        ObsFileUtil.editFile(dto.getOuterBoxSelfAdhesiveStickerFile(), dto.getId(), 1);
    }

    @Override
    public List<OrderPackageBomVo> getSkuSpecPackageBomList(SkuSpecPackageBomDto dto) {
        List<SkuSpecPriceDto> skuSpecList = dto.getSkuSpecList();
        List<Long> skuSpecIds = skuSpecList.stream().map(SkuSpecPriceDto::getSkuSpecId).collect(Collectors.toList());
        Map<Long, List<SkuSpecLink>> skuSpecMap = skuSpecLinkService.mapKGroup(
                SkuSpecLink::getSkuSpecId,
                q -> q.in(SkuSpecLink::getSkuSpecId, skuSpecIds).eq(SkuSpecLink::getType, 2));
        if (ObjectUtil.isEmpty(skuSpecMap)) {
            return Collections.emptyList();
        }
        List<Long> bomSpecIds = skuSpecMap.values().stream()
                .flatMap(item -> item.stream().map(SkuSpecLink::getBomSpecId))
                .collect(Collectors.toList());
        Map<Long, BomSpec> bomSpecPriceMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIds));

        List<OrderPackageBomVo> list = new ArrayList<>();
        for (SkuSpecPriceDto skuSpecPriceDto : skuSpecList) {
            List<SkuSpecLink> skuSpecLinkList = skuSpecMap.getOrDefault(skuSpecPriceDto.getSkuSpecId(), Collections.emptyList());
            List<OrderPackageBomVo> packageBomVoList = skuSpecLinkList.stream().map(item -> {
                BomSpec bomSpec = bomSpecPriceMap.get(item.getBomSpecId());
                OrderPackageBomVo orderPackageBomVo = new OrderPackageBomVo();
                orderPackageBomVo.setBomSpecName(bomSpec.getName());
                orderPackageBomVo.setBomSpecCode(bomSpec.getCode());
                orderPackageBomVo.setCostPrice(bomSpec.getCostPrice());
                orderPackageBomVo.setInternalSellingPrice(bomSpec.getInternalSellingPrice());
                orderPackageBomVo.setBomSpecId(item.getBomSpecId());
                orderPackageBomVo.setQuantity(item.getQuantity().multiply(skuSpecPriceDto.getQuantity()));
                return orderPackageBomVo;
            }).collect(Collectors.toList());
            list.addAll(packageBomVoList);
        }
        // 合并相同bom
        Map<Long, OrderPackageBomVo> bomVoMap = list.stream().collect(Collectors.toMap(
                OrderPackageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }));
        return new ArrayList<>(bomVoMap.values());
    }

    @Override
    public Page<OrderInfoVo> deletedOrderPage(OrderInfoSelectDto dto) {
        return Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)
                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)
                .orderByDesc(OrderInfo::getId)
                .eq(Department::getId, dto.getDepartmentId())
                .like(OrderInfo::getCode, dto.getCode())
                .like(OrderInfo::getWlnCode, dto.getWlnCode())
                .eq(OrderInfo::getDelFlag, StatusConstant.DELETED)
                .page(dto);
    }

    @LogicIgnore
    @Override
    public OrderInfoVo deletedOrderDetail(BaseSelectDto dto) {
        return detail(dto.getId());
    }

    @Override
    public void orderExportExcel(Long id) {
        OrderInfo orderInfo = this.getById(id);
        List<OrderSku> list = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<Long> bomSpecIds = list.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        List<Long> skuSpecIds = list.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(bomSpecIds);
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIds);
        List<OrderSku> orderPackageSkuList = list.stream()
                .filter(item -> !Objects.equals(bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());
        List<OrderSku> orderSkuList = list.stream()
                .filter(item -> Objects.equals(bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());
        BigDecimal orderPackageSku = orderPackageSkuList.stream()
                .map(item -> (item.getUnitPrice()
                        .add(item.getDeliveryMaterialsFee())
                        .add(item.getPackagingMaterialCost()))
                        .multiply(item.getQuantity()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal orderPackagePrice = orderPackageSku.divide(BigDecimal.valueOf(orderSkuList.size()), 2, RoundingMode.HALF_UP);
        List<OrderSkuExportVo> exportVoList = orderSkuList.stream().map(item -> {
            BigDecimal subtotal = item.getUnitPrice()
                    .add(item.getDeliveryMaterialsFee())
                    .add(item.getPackagingMaterialCost());
            BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
            SkuSpec skuSpec = skuSpecMap.get(item.getSkuSpecId());
            OrderSkuExportVo orderSkuExportVo = new OrderSkuExportVo();
            orderSkuExportVo.setWlnCode(orderInfo.getWlnCode());
            orderSkuExportVo.setSkuSpecCode(skuSpec.getCode());
            orderSkuExportVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
            orderSkuExportVo.setErpCode(item.getErpCode());
            orderSkuExportVo.setFeatureCode(item.getFeatureCode());
            orderSkuExportVo.setQuantity(item.getQuantity());
            orderSkuExportVo.setTotalAmount(subtotal.multiply(item.getQuantity()).add(orderPackagePrice));
            return orderSkuExportVo;
        }).collect(Collectors.toList());
        ExcelUtil.export(response, exportVoList, OrderSkuExportVo.class);
    }

    /**
     * 主材出库
     */
    private void semiFinishedProductOutStorage(List<InOutStorageBom> inOutStorageBomList, Map<Long, BomSpecBo> bomSpecBoMap, OrderInfo orderInfo) {
        List<InOutStorageBom> semiFinishedProductInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return false;
                    }
                    return ObjectUtil.equal(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto semiFinishedProductInOutStorageDto = new InOutStorageDto();
        semiFinishedProductInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        semiFinishedProductInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        semiFinishedProductInOutStorageDto.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        semiFinishedProductInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        semiFinishedProductInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        semiFinishedProductInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 主材退料入库");
        semiFinishedProductInOutStorageDto.setInOutStorageBomList(semiFinishedProductInOutStorageBomList);
        inOutStorageService.add(semiFinishedProductInOutStorageDto);
    }

    /**
     * 包材出库
     */
    private void packagingMaterialOutStorage(List<InOutStorageBom> inOutStorageBomList, Map<Long, BomSpecBo> bomSpecBoMap, OrderInfo orderInfo) {
        List<InOutStorageBom> packagingMaterialInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return true;
                    }
                    return ObjectUtil.notEqual(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
        packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
        packagingMaterialInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        packagingMaterialInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 包材退料入库");
        packagingMaterialInOutStorageDto.setInOutStorageBomList(packagingMaterialInOutStorageBomList);
        inOutStorageService.add(packagingMaterialInOutStorageDto);
    }

    private void sendMq(OrderInfoDto dto) {
        List<OrderSkuDto> orderSkuList = dto.getOrderSkuList();
        for (OrderSkuDto orderSkuDto : orderSkuList) {
            if (orderSkuDto.getArtworkLibraryId() == null || orderSkuDto.getArtworkLibraryId() == 0L) {
                continue;
            }
            TempArtworkMessage tempArtworkMessage = new TempArtworkMessage();
            tempArtworkMessage.setImgUrl(orderSkuDto.getBlueprint());
            tempArtworkMessage.setFileUrl(orderSkuDto.getProductionDocument());
            RabbitMqUtil.send(ArtworkConfig.DIRECT_EXCHANGE_NAME, ArtworkConfig.TEMP_ARTWORK_QUEUE_NAME, tempArtworkMessage);
        }
    }

    /**
     * 订单修改快递包材出入库操作
     */
    private void orderPackageBomInOutStorage(OrderInfoDto dto) {
        List<OrderPackageBom> oldOrderPackageBomList = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, dto.getId()));
        List<OrderPackageBomDto> newOrderPackageBomList = dto.getOrderPackageBomList();

        // 旧bom
        Map<Long, BigDecimal> oldBomMap = oldOrderPackageBomList.stream()
                .collect(Collectors.toMap(
                        OrderPackageBom::getBomSpecId,
                        OrderPackageBom::getQuantity,
                        BigDecimal::add
                ));
        Map<Long, BigDecimal> newBomMap = newOrderPackageBomList.stream()
                .collect(Collectors.toMap(
                        OrderPackageBom::getBomSpecId,
                        OrderPackageBom::getQuantity,
                        BigDecimal::add
                ));
        // 查询所有bom进行对比
        Set<Long> bomIds = new HashSet<>(oldBomMap.keySet());
        bomIds.addAll(newBomMap.keySet());
        Map<Long, BigDecimal> outBomMap = new HashMap<>();
        Map<Long, BigDecimal> inBomMap = new HashMap<>();
        for (Long bomId : bomIds) {
            BigDecimal oldReturnQuantity = oldBomMap.get(bomId);
            BigDecimal newReturnQuantity = newBomMap.get(bomId);
            // 旧bom没有，出库库存，反之则归还库存
            if (oldReturnQuantity == null) {
                outBomMap.put(bomId, newReturnQuantity);
                continue;
            } else if (newReturnQuantity == null) {
                inBomMap.put(bomId, oldReturnQuantity);
                continue;
            }
            // 都有值，判断数量
            if (oldReturnQuantity.compareTo(newReturnQuantity) == 0) {
                continue;
            }
            // 旧bom大于新bom则归还入库，反之出库库存
            if (oldReturnQuantity.compareTo(newReturnQuantity) > 0) {
                inBomMap.put(bomId, oldReturnQuantity.subtract(newReturnQuantity));
            } else {
                outBomMap.put(bomId, newReturnQuantity.subtract(oldReturnQuantity));
            }
        }
        if (!outBomMap.isEmpty()) {
            // 包材从包材仓出库
            List<InOutStorageBom> outStorageBomList = outBomMap.entrySet().stream()
                    .map(item -> {
                        InOutStorageBom inOutStorageBom = new InOutStorageBom();
                        inOutStorageBom.setBomSpecId(item.getKey());
                        inOutStorageBom.setQuantity(item.getValue());
                        return inOutStorageBom;
                    })
                    .collect(Collectors.toList());
            InOutStorageDto inOutStorageDto = new InOutStorageDto();
            inOutStorageDto.setType(InOutTypeEnum.OUT.getKey());
            inOutStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
            inOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            inOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            inOutStorageDto.setInOutStorageBomList(outStorageBomList);
            inOutStorageService.add(inOutStorageDto);
        }
        if (!inBomMap.isEmpty()) {
            // 包材归还入库
            List<InOutStorageBom> inOutStorageBomList = inBomMap.entrySet()
                    .stream()
                    .map(item -> {
                        InOutStorageBom inOutStorageBom = new InOutStorageBom();
                        inOutStorageBom.setBomSpecId(item.getKey());
                        inOutStorageBom.setQuantity(item.getValue());
                        return inOutStorageBom;
                    })
                    .collect(Collectors.toList());
            InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
            packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
            packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
            packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            packagingMaterialInOutStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
            packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            packagingMaterialInOutStorageDto.setRemark("订单产品快递包装修改：" + dto.getCode()
                    + (StrUtil.isBlank(dto.getWlnCode()) ? StringPool.EMPTY : "（" + dto.getWlnCode() + "）")
                    + " 包材归还入库");
            packagingMaterialInOutStorageDto.setInOutStorageBomList(inOutStorageBomList);
            inOutStorageService.add(packagingMaterialInOutStorageDto);
        }
    }

}
