package com.sd.business.service.order.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.dto.OrderSkuSelectDto;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.order.OrderSkuMapper;
import com.sd.business.service.order.OrderSkuService;
import com.sd.framework.util.sql.Sql;
import org.springframework.stereotype.Service;


/**
 * <p>
 * 订单sku 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class OrderSkuServiceImpl extends ServiceImpl<OrderSkuMapper, OrderSku> implements OrderSkuService {

    @Override
    public Page<OrderSkuVo> getPage(OrderSkuSelectDto dto) {

        return Sql.create(OrderSkuVo.class)
                .selectAll(OrderSku.class)
                .selectAs(OrderInfo::getCode, OrderSkuVo::getOrderCode)
                .selectAs(OrderInfo::getWlnCode, OrderSkuVo::getOrderWlnCode)
                .selectAs(OrderInfo::getStatus, OrderSkuVo::getOrderStatus)
                .selectAs(OrderInfo::getSettlementStatus, OrderSkuVo::getOrderSettlementStatus)
                .selectAs(OrderInfo::getTotalAmount, OrderSkuVo::getOrderTotalAmount)
                .selectAs(OrderInfo::getWlnCreateTime, OrderSkuVo::getWlnCreateTime)
                .selectAs(SkuSpec::getCode, OrderSkuVo::getSkuSpecCode)
                .selectAs(SkuSpec::getName, OrderSkuVo::getSkuSpecName)
                .selectAs(BomSpec::getCode, OrderSkuVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuVo::getBomSpecName)
                .selectAs(Department::getName, OrderSkuVo::getDepartmentName)

                .from(OrderSku.class)
                .leftJoin(OrderInfo.class, OrderSku::getOrderId)
                .leftJoin(SkuSpec.class, OrderSku::getSkuSpecId)
                .leftJoin(BomSpec.class, OrderSku::getBomSpecId)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)

                .orderByDesc(OrderSku::getId)
                .eq(OrderInfo::getStatus, dto.getOrderStatus())
                .eq(OrderInfo::getDepartmentId, dto.getDepartmentId())
                .ge(OrderInfo::getWlnCreateTime, dto.getBeginTime())
                .le(OrderInfo::getWlnCreateTime, dto.getEndTime())
                .eq(OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())
                .like(OrderInfo::getCode, dto.getOrderCode())
                .like(OrderInfo::getWlnCode, dto.getOrderWlnCode())
                .like(SkuSpec::getCode, dto.getSkuSpecCode())
                .like(SkuSpec::getName, dto.getSkuSpecName())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getName, dto.getBomSpecName())
                .like(Department::getName, dto.getDepartmentName())
                .page(dto);
    }

}
