package com.sd.business.service.price.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.price.dto.PriceBillingStandardDto;
import com.sd.business.entity.price.dto.PriceBillingStandardSelectDto;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.price.vo.PriceBillingStandardVo;
import com.sd.business.mapper.price.PriceBillingStandardMapper;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * <p>
 * 加工计费标准 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class PriceBillingStandardServiceImpl extends ServiceImpl<PriceBillingStandardMapper, PriceBillingStandard> implements PriceBillingStandardService {

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Override
    public Page<PriceBillingStandardVo> getPage(PriceBillingStandardSelectDto dto) {

        Page<PriceBillingStandardVo> page = Sql.create(PriceBillingStandardVo.class)
                .selectAll(PriceBillingStandard.class)
                .from(PriceBillingStandard.class)
                .orderByDesc(PriceBillingStandard::getId)
                .eq(PriceBillingStandard::getName, dto.getName())
                .page(dto);

        return page;
    }

    @Override
    public PriceBillingStandardVo detail(Long id) {

        PriceBillingStandardVo vo = Sql.create(PriceBillingStandardVo.class)
                .selectAll(PriceBillingStandard.class)
                .from(PriceBillingStandard.class)
                .eq(PriceBillingStandard::getId, id)
                .one();

        Assert.notNull(vo, "未知加工计费标准");

        List<PriceBillingStandardDetail> priceBillingStandardDetailList = priceBillingStandardDetailService.list(q -> q
                .eq(PriceBillingStandardDetail::getPriceBillingStandardId, id)
                .orderByAsc(PriceBillingStandardDetail::getSort)
        );

        vo.setPriceBillingStandardDetailList(priceBillingStandardDetailList);
        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PriceBillingStandardDto dto) {
        save(dto);
        List<PriceBillingStandardDetail> list = dto.getPriceBillingStandardDetailList();
        list.forEach(item -> item.setPriceBillingStandardId(dto.getId()));
        priceBillingStandardDetailService.saveBatch(list);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PriceBillingStandardDto dto) {
        updateById(dto);

        List<PriceBillingStandardDetail> list = dto.getPriceBillingStandardDetailList();
        list.forEach(item -> item.setPriceBillingStandardId(dto.getId()));
        priceBillingStandardDetailService.editLinked(list, PriceBillingStandardDetail::getPriceBillingStandardId, dto.getId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        removeById(id);
        priceBillingStandardDetailService.remove(q -> q.eq(PriceBillingStandardDetail::getPriceBillingStandardId, id));
    }

}
