package com.sd.business.util.fixation;

import com.sd.business.entity.work.constant.MaterialsConstant;
import lombok.Getter;

import java.math.BigDecimal;
import java.math.RoundingMode;

public abstract class FixationStrategy {

    /**
     * 卷材长度
     */
    @Getter
    protected final BigDecimal masterLength = new BigDecimal(MaterialsConstant.MASTER_LENGTH);

    /**
     * 卷材宽度
     */
    @Getter
    protected final BigDecimal masterWidth = new BigDecimal(MaterialsConstant.MASTER_WIDTH);

    /**
     * 卷材面积
     */
    @Getter
    protected final BigDecimal masterArea = masterLength.multiply(masterWidth);

    /**
     * 已使用面积
     */
    @Getter
    protected BigDecimal usedArea = BigDecimal.ZERO;

    /**
     * 排单数量
     */
    @Getter
    protected BigDecimal schedulingNum = BigDecimal.ZERO;

    /**
     * 最大长度
     */
    @Getter
    protected BigDecimal maxLength = BigDecimal.ZERO;

    /**
     * 排版参数
     */
    @Getter
    protected CoordinateBo coordinateBo;

    /**
     * 是否可以继续排版
     */
    protected Boolean hasNext = true;

    /**
     * 获取下一个排版参数
     */
    public abstract CoordinateBo next();

    /**
     * 是否可以继续排版
     */
    public final boolean hasNext() {
        return this.hasNext;
    }

    /**
     * 获取使用率
     */
    public final BigDecimal getUseRatio() {
        return usedArea.multiply(new BigDecimal(100)).divide(masterArea, 2, RoundingMode.HALF_UP);
    }

}
