package com.sd.business.util.fixation;

import com.sd.business.entity.work.constant.MaterialsConstant;
import com.sd.business.util.fixation.strategy.Fixation_30_40;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

import java.util.Random;

public class Test extends javafx.application.Application {

    private static final FixationStrategy fixationStrategy = new Fixation_30_40();

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        AnchorPane pane = new AnchorPane();
        Canvas canvas = new Canvas(MaterialsConstant.MASTER_WIDTH, MaterialsConstant.MASTER_LENGTH);
        pane.getChildren().add(canvas);
        canvas.relocate(100, 100);
        // 绘制最外层的矩形
        draw(canvas, 0, 0, MaterialsConstant.MASTER_WIDTH, MaterialsConstant.MASTER_LENGTH, true);
        // 添加按钮
        Button nextButton = new Button("Next");
        nextButton.setPrefSize(80, 50);

        nextButton.setOnAction(actionEvent -> {
            try {

                CoordinateBo next = fixationStrategy.next();

                draw(canvas,
                        next.getX().doubleValue(),
                        next.getY().doubleValue(),
                        next.getBleedingWidth().doubleValue(),
                        next.getBleedingLength().doubleValue(),
                        false);

            } catch (Exception e) {
                Alert alert = new Alert(Alert.AlertType.WARNING);
                alert.setContentText("已经没有可以放置的矩形了！");
                alert.showAndWait();
            }
        });
        pane.getChildren().add(nextButton);
        primaryStage.setTitle("二维矩形装箱可视化");
        primaryStage.setScene(new Scene(pane, MaterialsConstant.MASTER_WIDTH + 200, MaterialsConstant.MASTER_LENGTH + 200, Color.AQUA));
        primaryStage.show();
    }

    private void draw(Canvas canvas, double x, double y, double l, double w, boolean isBound) {
        GraphicsContext gc = canvas.getGraphicsContext2D();
        // 边框
        gc.setStroke(Color.BLACK);
        gc.setLineWidth(2);
        gc.strokeRect(x, y, l, w);
        // 填充
        if (!isBound) {
            gc.setFill(new Color(new Random().nextDouble(), new Random().nextDouble(), new Random().nextDouble(), new Random().nextDouble()));
        } else {
            gc.setFill(new Color(1, 1, 1, 1));
        }
        gc.fillRect(x, y, l, w);
    }

}
