package com.sd.business.util.packing.model;

import com.sd.business.util.packing.entity.Solution;
import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
public class Genome {

    /**
     * 边界的长
     */
    private double length;

    /**
     * 边界的宽
     */
    private double width;

    /**
     * 矩形数组
     */
    private Item[] items;

    /**
     * 是否可以旋转
     */
    private boolean isRotateEnable;

    /**
     * 基因序列
     */
    private int[] genomeArray;

    /**
     * 适应度函数值(在二维装箱问题中，其实就是装载利用率)
     */
    private double fitness;

    /**
     * 序列对应的装载结果
     */
    private Solution solution;

    /**
     * 基因对象的构造函数
     *
     * @param length         边界高度
     * @param width          边界宽度
     * @param items          矩形集合
     * @param isRotateEnable 是否可以旋转
     * @param genomeArray    基因序列
     */
    public Genome(double length, double width, Item[] items, boolean isRotateEnable, int[] genomeArray) {
        this.width = width;
        this.length = length;
        this.items = items;
        this.isRotateEnable = isRotateEnable;
        this.genomeArray = genomeArray;
    }

    /**
     * 获取适应值和路径长度
     */
    public void updateFitnessAndSolution() {
        Item[] items = new Item[this.items.length];
        for (int i = 0; i < genomeArray.length; i++) {
            items[i] = this.items[genomeArray[i]];
        }
        solution = new SkyLinePacking(isRotateEnable, length, width, items).packing();
        fitness = solution.getUseRatio();
    }

}
